/*
 * Copyright (c) 2014-2015, Siemens AG. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef EMBB_DATAFLOW_INTERNAL_ACTION_H_
#define EMBB_DATAFLOW_INTERNAL_ACTION_H_

#include <cstddef>

#include <embb/tasks/task_context.h>

#include <embb/dataflow/internal/node.h>

namespace embb {
namespace dataflow {
namespace internal {

class Action {
 public:
  Action() : node_(NULL), clock_(0), pending_(0) {}
  Action(Node * node, int clock) : node_(node), clock_(clock), pending_(2) {}

  void RunSequential() {
    pending_ = 1;
    node_->Run(clock_);
    pending_ = 0;
  }

  void RunMTAPI(embb::tasks::TaskContext & /*context*/) {
    pending_ = 1;
    node_->Run(clock_);
    pending_ = 0;
  }

  bool IsPending() const { return pending_ > 0; }

  int GetClock() const { return clock_; }

 private:
  Node * node_;
  int clock_;
  volatile int pending_;
};

} // namespace internal
} // namespace dataflow
} // namespace embb

#endif // EMBB_DATAFLOW_INTERNAL_ACTION_H_
